@@ -1,28 +1,36 @@
+""" Upload Dataset Mixin """
+
 import os
 import warnings
+from typing import Union, Dict
+from datetime import datetime
 from concurrent.futures.thread import ThreadPoolExecutor
-from typing import Union, List, Dict
 
-import lightly_utils.image_processing
-from torch.utils import data
 import tqdm
-
-from lightly.openapi_generated.swagger_client import TagCreator, SamplesApi, SampleWriteUrls, SampleData
-from lightly.openapi_generated.swagger_client.models.sample_create_request import SampleCreateRequest
-from lightly.api.utils import check_filename, PIL_to_bytes, MAXIMUM_FILENAME_LENGTH
-from lightly.openapi_generated.swagger_client.models.initial_tag_create_request import InitialTagCreateRequest
-from lightly.openapi_generated.swagger_client.models.job_status_meta import JobStatusMeta
-from lightly.openapi_generated.swagger_client.models.job_status_upload_method import JobStatusUploadMethod
-
-from lightly.data.dataset import LightlyDataset
-
-from lightly.api.utils import retry
-
 from lightly_utils import image_processing
 
+from lightly.data.dataset import LightlyDataset
+from lightly.api.utils import check_filename
+from lightly.api.utils import MAXIMUM_FILENAME_LENGTH
+from lightly.api.utils import retry
+from lightly.openapi_generated.swagger_client import TagCreator
+from lightly.openapi_generated.swagger_client import SampleWriteUrls
+from lightly.openapi_generated.swagger_client.models.sample_create_request \
+    import SampleCreateRequest
+from lightly.openapi_generated.swagger_client.models.tag_upsize_request \
+    import TagUpsizeRequest
+from lightly.openapi_generated.swagger_client.models.initial_tag_create_request\
+    import InitialTagCreateRequest
+from lightly.openapi_generated.swagger_client.models.job_status_meta \
+    import JobStatusMeta
+from lightly.openapi_generated.swagger_client.models.job_status_upload_method \
+    import JobStatusUploadMethod
 
 
 class _UploadDatasetMixin:
+    """Mixin to upload datasets to the Lightly Api.
+
+    """
 
     def upload_dataset(self,
                        input: Union[str, LightlyDataset],
@@ -43,35 +51,33 @@ class _UploadDatasetMixin:
                 Maximum number of requests a single worker can do before he has
                 to wait for the others.
             mode:
-                One of [full, thumbnails, metadata]. Whether to upload thumbnails,
-                full images, or metadata only.
+                One of [full, thumbnails, metadata]. Whether to upload
+                thumbnails, full images, or metadata only.
 
         Raises:
             ValueError if dataset is too large or input has the wrong type
             RuntimeError if the connection to the server failed.
 
         """
-        no_tags_on_server = len(self._get_all_tags())
-        if no_tags_on_server > 0:
-            warnings.warn(f"Dataset with id {self.dataset_id} has already been completely uploaded to the platform. Skipping upload.")
-            return
 
-        # Check input variable 'input'
+        # get all tags of the dataset
+        tags = self._get_all_tags()
+        if len(tags) > 0:
+            print(
+                f'Dataset with id {self.dataset_id} has {len(tags)} tags.',
+                flush=True
+            )
+
+        # parse "input" variable
         if isinstance(input, str):
             dataset = LightlyDataset(input_dir=input)
         elif isinstance(input, LightlyDataset):
             dataset = input
         else:
-            raise ValueError(f"input must either be a LightlyDataset or the path to the dataset as str, "
-                             f"but is of type {type(input)}")
-
-        # check the allowed dataset size
-        max_dataset_size_str = self.quota_api.get_quota_maximum_dataset_size()
-        max_dataset_size = int(max_dataset_size_str)
-        if len(dataset) > max_dataset_size:
-            msg = f'Your dataset has {len(dataset)} samples which'
-            msg += f' is more than the allowed maximum of {max_dataset_size}'
-            raise ValueError(msg)
+            raise ValueError(
+                f'input must either be a LightlyDataset or the path to the'
+                f'dataset as str, but has type {type(input)}'
+            )
 
         # handle the case where len(dataset) < max_workers
         max_workers = min(len(dataset), max_workers)
@@ -79,19 +85,37 @@ class _UploadDatasetMixin:
 
         # upload the samples
         if verbose:
-            print(f'Uploading images (with {max_workers} workers).', flush=True)
+            print(
+                f'Uploading images (with {max_workers} workers).',
+                flush=True
+            )
 
-
-        # calculate the files size more efficiently
-        lightly_utils.image_processing.metadata._size_in_bytes = lambda img: 0
+        # TODO: remove _size_in_bytes from image_processing
+        image_processing.metadata._size_in_bytes = \
+            lambda img: 0 # pylint: disable=protected-access
 
         # get the filenames of the samples already on the server
-        self.samples_api: SamplesApi
-        samples: List[SampleData] = self.samples_api.get_samples_by_dataset_id(dataset_id=self.dataset_id)
-        filenames = [sample.file_name for sample in samples]
-        if len(filenames) > 0:
-            print(f"Found {len(filenames)} images already on the server, they are skipped during the upload.")
-        filenames_set = set(filenames)
+        samples = self.samples_api.get_samples_by_dataset_id(
+            dataset_id=self.dataset_id
+        )
+        filenames_on_server = [sample.file_name for sample in samples]
+        filenames_on_server_set = set(filenames_on_server)
+        if len(filenames_on_server) > 0:
+            print(
+                f'Found {len(filenames_on_server)} images already on the server'
+                ', they are skipped during the upload.'
+            )
+
+        # check the maximum allowed dataset size
+        total_filenames = set(dataset.get_filenames()).union(
+            filenames_on_server_set
+        )
+        max_dataset_size = \
+            int(self.quota_api.get_quota_maximum_dataset_size())
+        if len(total_filenames) > max_dataset_size:
+            msg = f'Your dataset has {len(dataset)} samples which'
+            msg += f' is more than the allowed maximum of {max_dataset_size}'
+            raise ValueError(msg)
 
         # index custom metadata by filename (only if it exists)
         filename_to_metadata = {}
@@ -103,29 +127,28 @@ class _UploadDatasetMixin:
             )
 
         # register dataset upload
-        try:
-            job_status_meta = JobStatusMeta(
-                total=len(dataset),
-                processed=len(filenames),
-                is_registered=True,
-                upload_method=JobStatusUploadMethod.USER_PIP,
-            )
-            self.datasets_api.register_dataset_upload_by_id(
-                job_status_meta,
-                self.dataset_id
-            )
-        except Exception:
-            # TODO: remove try/catch after release of latest API
-            pass
+        job_status_meta = JobStatusMeta(
+            total=len(total_filenames),
+            processed=len(filenames_on_server),
+            is_registered=True,
+            upload_method=JobStatusUploadMethod.USER_PIP,
+        )
+        self.datasets_api.register_dataset_upload_by_id(
+            job_status_meta,
+            self.dataset_id
+        )
 
-        pbar = tqdm.tqdm(unit='imgs', total=len(dataset)-len(filenames))
+        pbar = tqdm.tqdm(
+            unit='imgs',
+            total=len(total_filenames) - len(filenames_on_server),
+        )
         tqdm_lock = tqdm.tqdm.get_lock()
 
         # define lambda function for concurrent upload
         def lambda_(i):
             # load image
-            image, label, filename = dataset[i]
-            if filename in filenames_set:
+            image, _, filename = dataset[i]
+            if filename in filenames_on_server_set:
                 # the sample was already uploaded
                 return True
 
@@ -138,21 +161,22 @@ class _UploadDatasetMixin:
             try:
                 self._upload_single_image(
                     image=image,
-                    label=label,
                     filename=filename,
                     filepath=filepath,
                     mode=mode,
                     custom_metadata=custom_metadata_item,
                 )
                 success = True
-            except Exception as e:
-                warnings.warn(f"Upload of image {filename} failed with error {e}")
+            except Exception as e: # pylint: disable=broad-except
+                warnings.warn(
+                    f'Upload of image {filename} failed with error {e}'
+                )
                 success = False
 
             # update the progress bar
-            tqdm_lock.acquire()  # lock
-            pbar.update(1)  # update
-            tqdm_lock.release()  # unlock
+            tqdm_lock.acquire()
+            pbar.update(1)
+            tqdm_lock.release()
             # return whether the upload was successful
             return success
 
@@ -174,12 +198,29 @@ class _UploadDatasetMixin:
         else:
             img_type = 'meta'
 
-        initial_tag_create_request = InitialTagCreateRequest(img_type=img_type, creator=TagCreator.USER_PIP)
-        self.tags_api.create_initial_tag_by_dataset_id(body=initial_tag_create_request, dataset_id=self.dataset_id)
+        if len(tags) == 0:
+            # create initial tag
+            initial_tag_create_request = InitialTagCreateRequest(
+                img_type=img_type,
+                creator=TagCreator.USER_PIP
+            )
+            self.tags_api.create_initial_tag_by_dataset_id(
+                body=initial_tag_create_request,
+                dataset_id=self.dataset_id,
+            )
+        else:
+            # upsize existing tags
+            upsize_tags_request = TagUpsizeRequest(
+                upsize_tag_name=datetime.now().strftime('%Y%m%d_%Hh%Mm%Ss'),
+                upsize_tag_creator=TagCreator.USER_PIP,
+            )
+            self.tags_api.upsize_tags_by_dataset_id(
+                body=upsize_tags_request,
+                dataset_id=self.dataset_id,
+            )
 
     def _upload_single_image(self,
                              image,
-                             label: int,
                              filename: str,
                              filepath: str,
                              mode: str,
@@ -187,27 +228,25 @@ class _UploadDatasetMixin:
         """Uploads a single image to the Lightly platform.
 
         """
-        self.samples_api: SamplesApi
-
         # check whether the filepath is too long
         if not check_filename(filepath):
-            msg = (f'Filepath {filepath} is longer than the allowed maximum of {MAXIMUM_FILENAME_LENGTH}'
-                   'characters and will be skipped.')
+            msg = ('Filepath {filepath} is longer than the allowed maximum of '
+                   f'{MAXIMUM_FILENAME_LENGTH} characters and will be skipped.')
             raise ValueError(msg)
 
         # calculate metadata, and check if corrupted
         metadata = image_processing.Metadata(image).to_dict()
-        metadata["sizeInBytes"] = os.path.getsize(filepath)
+        metadata['sizeInBytes'] = os.path.getsize(filepath)
 
         # try to get exif data
         try:
             exifdata = image_processing.Exifdata(image)
-        except Exception:
+        except Exception: # pylint disable=broad-except
             exifdata = None
 
         # generate thumbnail if necessary
         thumbname = None
-        if not metadata['is_corrupted'] and mode in ["thumbnails", "full"]:
+        if not metadata['is_corrupted'] and mode in ['thumbnails', 'full']:
             thumbname = '.'.join(filename.split('.')[:-1]) + '_thumb.webp'
 
         body = SampleCreateRequest(
@@ -223,7 +262,7 @@ class _UploadDatasetMixin:
             dataset_id=self.dataset_id
         ).id
 
-        if not metadata['is_corrupted'] and mode in ["thumbnails", "full"]:
+        if not metadata['is_corrupted'] and mode in ['thumbnails', 'full']:
 
             def upload_thumbnail(image, signed_url):
                 thumbnail = image_processing.Thumbnail(image)
@@ -243,7 +282,7 @@ class _UploadDatasetMixin:
                         signed_url
                     )
 
-            if mode == "thumbnails":
+            if mode == 'thumbnails':
                 thumbnail_url = retry(
                     self.samples_api.get_sample_image_write_url_by_id,
                     dataset_id=self.dataset_id,
@@ -251,15 +290,13 @@ class _UploadDatasetMixin:
                     is_thumbnail=True
                 )
                 upload_thumbnail(image, thumbnail_url)
-            elif mode == "full":
+            elif mode == 'full':
                 sample_write_urls: SampleWriteUrls = retry(
-                    self.samples_api.get_sample_image_write_urls_by_id, dataset_id=self.dataset_id, sample_id=sample_id
+                    self.samples_api.get_sample_image_write_urls_by_id,
+                    dataset_id=self.dataset_id,
+                    sample_id=sample_id
                 )
                 upload_thumbnail(image, sample_write_urls.thumb)
                 upload_full_image(filepath, sample_write_urls.full)
 
-
-
         image.close()
-
-
